(** Signature of wells. *)

type url = string

type contents = Source.action_arg list list  (* list of action args of the objects to be exported by the well. *)

module type T =
  sig
    type t  (** Type of wells. *)

    val kinds : contents -> (string * string) list
	(** [kinds ()] returns a list of pairs [(ext,label)] where [ext] is a file extension, and [label] a description of exported kind. *)
    val action_args : t -> Source.action_arg list
	(** [action_args wll] returns a list of argument for actions on the well [wll]. *)

    val init : t -> contents -> unit
	(** [init wll cs] initializes the well [wll] as the contents [cs] by executing the first export. *)
    val update : t -> contents -> unit
	(** [update wll cs] updates the well [wll] by executing a subsequent export. *)
    val final : t -> contents -> unit  (* remove the production of the well *)
	(** [final wll cs] removes the well [wll] by deleting its exports. *)
    val move : t -> url -> unit
	(** [move wll url] relocates the well [wll] at the location [url]. *)

    val make : url -> t  (* creates a new well from a url *)
    val url : t -> url  (* returns the url defining a well *)
  end
