(** Source: personal agenda files (.agd). *)

open Source

module Param =
  struct
    let filter url = Filename.check_suffix url ".agd"

    let may_have_changed_since = Url.may_have_changed_since

    let stringint_of_month = function
      | "jan" -> "01"
      | "feb" -> "02"
      | "mar" -> "03"
      | "apr" -> "04"
      | "may" -> "05"
      | "jun" -> "06"
      | "jul" -> "07"
      | "aug" -> "08"
      | "sep" -> "09"
      | "oct" -> "10"
      | "nov" -> "11"
      | "dec" -> "12"
      | _ -> "00"


    let get_objs url proper_parts =
      let lf_src = Url.fields_of_url "source" [] url in
      let src_pre = Url.preview_of_url url in
      let src_args = {mime=Url.mime url "text/agd"; role=""; location=url}::Url.args_of_url url in
      let obj_src = ("",(lf_src, String src_pre, src_args)) in
      if proper_parts
      then
	let make_obj pos (weekday,day,month,year) time_opt item =
	  let ref = 
	    let h, m = Common.fold_option (fun hm -> hm) ("0","0") time_opt in
	    Printf.sprintf "%4s%2s%2s-%2s%2s-%2d" year (stringint_of_month month) day h m pos in
	  let lf =
	    ("",Fields.Date (int_of_string day, month, int_of_string year))::
	    ("event", Fields.StringList [item])::
	    lf_src in
	  let lf =
	    match time_opt with
	    | None -> lf
	    | Some (h,m) -> ("",Fields.Time ((int_of_string h) mod 24, int_of_string m, 0))::lf in
	  let pre =
	    match time_opt with
	    | None -> Printf.sprintf "%s %s %s %s\n%s" weekday day month year item
	    | Some (h,m) -> Printf.sprintf "%s %s %s %s - %s:%s\n%s" weekday day month year h m item in
	  let args = {mime="line"; role=string_of_int pos; location=string_of_int pos} :: src_args in
	  ref, (lf, String pre, args) in
	let add_obj pos date_opt time_opt item objs =
	  if item = ""
	  then objs
	  else
	    match date_opt with
	    | None -> objs
	    | Some date -> make_obj pos date time_opt item :: objs in
	let ch_in = open_in (Url.path_of_url url) in
	let objs, _, _, _, _ =
	  Common.fold_in_channel
	    (fun (objs,pos,date_opt,time_opt,item) line ->
	      if Str.string_match (Str.regexp "[ \t]*$") line 0 then
		(add_obj pos date_opt time_opt item objs, pos, date_opt, time_opt, "")
	      else if Str.string_match (Str.regexp "\\([A-Z][a-z][a-z]\\) +\\([1-9][0-9]?\\) +\\([a-z]+\\) +\\([0-9]+\\)") line 0 then
		let weekday = Str.matched_group 1 line in
		let day = Str.matched_group 2 line in
		let month = Str.matched_group 3 line in
		let year = Str.matched_group  4 line in
		(add_obj pos date_opt time_opt item objs, 0, Some (weekday,day,month,year), None, "")
	      else if Str.string_match (Str.regexp "\\([0-9][0-9]\\)\\([0-9][0-9]\\)") line 0 then
		let h = Str.matched_group 1 line in
		let m = Str.matched_group 2 line in
		(add_obj pos date_opt time_opt item objs, 0, date_opt, Some (h,m), "")
	      else if line = "matin" then
		(add_obj pos date_opt time_opt item objs, 0, date_opt, Some ("08","00"), "")
	      else if line = "midi" then
		(add_obj pos date_opt time_opt item objs, 0, date_opt, Some ("12","00"), "")
	      else if line = "aprem" then
		(add_obj pos date_opt time_opt item objs, 0, date_opt, Some ("14","00"), "")
	      else if line = "soir" then
		(add_obj pos date_opt time_opt item objs, 0, date_opt, Some ("19","00"), "")
	      else if Str.string_match (Str.regexp "[ \t]") line 0 then
		(objs, pos, date_opt, time_opt, item ^ "\n" ^ line)
	      else
		(add_obj pos date_opt time_opt item objs, pos+1, date_opt, time_opt, line))
	    ([obj_src], 0, None, None,  "")
	    ch_in in
	objs
      else [obj_src]
  end

module Make : Source.T =
  Setobj.Make(Param)

