(** Source: Camelis context files (.ctx) or executables (.exe) generating them (only 'mk' commands) *)

open Source

module Param =
  struct
    let filter url = Filename.check_suffix url ".ctx" || Filename.check_suffix url ".exe"

    let may_have_changed_since = Url.may_have_changed_since

    let parse_line dirname = parser
      | [<'Token.Ident "mk"; args = Source.parse_action_args; pre = Source.parse_preview dirname; str>] ->
	  let ref =
	    try
	      let arg = List.find (fun arg -> arg.mime = "ref") args in
	      arg.location
	    with Not_found ->
	      match pre with
	      | String s -> s
	      | Picture (_,s) -> s in
	  let d = [("", Fields.Verbatim (Syntax.stringizer (Syntax.list_of_stream str)))] in
	  (ref, (d, pre, args))

    let get_objs url proper_parts =
      let parts =
	if proper_parts
	then
	  let dirname = Filename.dirname url in
	  let ch_in =
	    if Filename.check_suffix url ".ctx"
	    then open_in (Url.path_of_url url)
	    else Unix.open_process_in url (* command *) in
	  let res =
	    Common.fold_in_channel
	      (fun res line ->
		try parse_line dirname (Syntax.from_string line) :: res
		with exn ->
		  prerr_string (Printexc.to_string exn); prerr_newline (); res)
	      [] ch_in in
	  close_in ch_in;
	  res
	else [] in
      let lf_src = Url.fields_of_url "source" [] url in
      let src_pre = Url.preview_of_url url in
      let src_arg =
	if Filename.check_suffix url ".ctx"
	then {mime=Url.mime url "text/ctx"; role=""; location=url}
	else {mime="exec"; role=""; location=url} in
      ("",(lf_src, String src_pre, src_arg::Url.args_of_url url))::parts
  
  end

module Make : Source.T =
  Setobj.Make (Param)
