(** Source: DBLP record. *)

let re_accents = Str.regexp "{\\\\['`\"^~=.]?\\(.\\)}"
let re_letters = Str.regexp "\\\\\\([ijoOlL]\\)"
let re_others  = Str.regexp "\\\\['`\"^~=.cuvHdbt]{\\(.\\)}"

let print_bibitem ch s =
  let regexp_bibitem1 = Str.regexp "<pre>@\\([^{]+\\){<a[^<]+</a>:\\([^,]+\\),\\([^<]*\\)</pre>" in
  try
    ignore (Str.search_forward regexp_bibitem1 s 0);
    let kind = Str.matched_group 1 s in
    let ref = Str.matched_group 2 s in
    let fields = Str.matched_group 3 s in
    let fields = Str.global_replace re_accents "\\1" fields in
    let fields = Str.global_replace re_letters "\\1" fields in
    let fields = Str.global_replace re_others  "\\1" fields in
    Printf.fprintf ch "@%s{%s,%s\n" kind ref fields
  with Not_found -> prerr_string "Dblp_rec: wrong format\n"

let bibtex_file path =
  let bibfile = (Filename.chop_suffix path ".html") ^ ".bib" in
  let ch = open_out bibfile in
  let html = Common.string_of_file path in
  print_bibitem ch html;
  close_out ch;
  bibfile

open Source

module Param =
  struct
    let filter =
      let re = Str.regexp_string "http://dblp.uni-trier.de/rec/bibtex/" in
      fun url -> Str.string_match re url 0

    let may_have_changed_since url _ = true

    let text_of_source url = bibtex_file (Url.path_of_url url)

    let wrapper url (lf,pre,args) =
      let lf_url = Url.fields_of_url "source" [] url in
      (lf_url @ lf, pre, {mime="url/text/html"; role="DBLP BibTeX record"; location=url}::args)
  end
module Make : T =
  Sglobj_lexyacc.Make(Param)(Bibtex_p)(Bibtex_l)

(*
open Source

module Param =
  struct
    let text_of_source (path : string) = bibtex_file path
    let whole_as_object = false
  end
module Make : Source.T = Urlparts_lexyacc.Make(Param)(Bibtex_p)(Bibtex_l)(
  struct
    let re = Str.regexp_string "http://dblp.uni-trier.de/rec/bibtex/"
    let filter url = Str.string_match re url 0
    let url_attributes = []
    let preview url ref pre = Source.String pre
    let main_arg url = {mime="url/text/html"; role="BibTeX item"; location=url}
  end)
*)
