(** Source: list of birds *)

let find_all_matched regexp s =
  let _, occs =
    Common.fold_while
      (fun (pos,res) ->
	try
	  ignore (Str.search_forward regexp s pos);
	  let pos' = Str.match_end () in
	  let occ = Str.matched_group 1 s in
	  Some (pos',occ::res)
	with Not_found -> None)
      (0,[]) in
  List.rev occs

let re = Str.regexp "\"\\(oiseaux\\.[^\"]+\\.html\\)\""

open Source

module Param =
  struct
    let filter =
      fun url -> url = "http://www.oiseaux.net/oiseaux/europe.html"

    let may_have_changed_since = Url.may_have_changed_since

    let get_srcs url recursive f =
      let html = Common.string_of_file (Url.path_of_url url) in
      let pays = find_all_matched re html in
      List.map
	(fun p -> prerr_endline p; "http://www.oiseaux.net/liste/" ^ p)
	pays

    let move_src _ src = src
  end
module Make : Source.T =
  Setsrc.Make(Param)

