
module Make
(Param : sig
  val filter : Source.url -> bool
  val may_have_changed_since : Source.url -> float -> bool
  val get_obj : string -> Fields.t * Source.preview * Source.action_arg list
end) : Source.T =
  struct
    include Source.Default

    type obj = {
	mutable preview : Source.preview;
	mutable action_args : Source.action_arg list;
	mutable digest : string
      }

    type t = {
	mutable url : string;
	mutable options : Source.src_option list;
	mutable last_update : float;
	mutable obj : obj option;
      }

    let ref = ""

    let make url =
      if Param.filter url
      then {
	url = url;
	options = [];
	last_update = 0.;
	obj = None;
      }
      else raise (Invalid_argument "Sglobj.make: invalid url")

    let url s = s.url

    let eq s1 s2 = s1.url = s2.url

    let parts s =
      match s.obj with
      | None -> []
      | Some _ -> [ref]

    let preview s _ =
      match s.obj with
      | None -> invalid_arg "Sglobj.preview: invalid ref"
      | Some obj -> obj.preview

    let action_args s _ =
      match s.obj with
      | None -> invalid_arg "Sglobj.action_args: invalid ref"
      | Some obj -> obj.action_args

    let init s options =
      s.options <- options;
      s.last_update <- Unix.time ();
      try
	let lf, pre, args = Param.get_obj s.url in
	let d = Fields.prints lf in
	let pre = Fields.preview_utf pre in
	let obj = {
	  preview = pre;
	  action_args = args;
	  digest = Digest.string (Syntax.stringizer d)} in
	s.obj <- Some obj;
	[ Source.Make (s.url, ref, d) ]
      with _ -> []

    let update s options =
      if s.options = options && not (Param.may_have_changed_since s.url s.last_update)
      then begin
	s.last_update <- Unix.time ();
	[] end
      else begin
	s.last_update <- Unix.time ();
	s.options <- options;
	try
	  let lf, pre, args = Param.get_obj s.url in
	  let d = Fields.prints lf in
	  let pre = Fields.preview_utf pre in
	  let digest = Digest.string (Syntax.stringizer d) in
	  match s.obj with
	  | None ->
	      let obj = {
		preview = pre;
		action_args = args;
		digest = digest} in
	      s.obj <- Some obj;
	      [ Source.Make (s.url, ref, d) ]
	  | Some obj ->
	      obj.preview <- pre;
	      obj.action_args <- args;
	      if obj.digest <> digest
	      then begin
		obj.digest <- digest;
		[ Source.UpdateIntrSrc (s.url, ref, d) ] end
	      else []
	with _ -> [] (* safety *)
      end

    let final s =
      match s.obj with
      | None -> []
      | Some obj -> [ Source.RemoveSrc (s.url, ref) ]

    let move s new_url =
      s.url <- new_url;
      []

  end
