
module Make
(Param : sig
  val filter : Source.url -> bool
  val may_have_changed_since : Source.url -> float -> bool
  val text_of_source : Source.url -> Source.url (* result is a file path *)
  val wrapper : Source.url -> Fields.t * Source.preview * Source.action_arg list -> Fields.t * Source.preview * Source.action_arg list
end)
(Parser : sig
  type token
  val token_start : token
  val start : (Lexing.lexbuf -> token) -> Lexing.lexbuf -> string * (Fields.t * Source.preview * Source.action_arg list)
end)
(Lexer : sig
  exception Eof
  val token : Lexing.lexbuf -> Parser.token
end) =
  Sglobj.Make
    (struct
      let filter = Param.filter

      let may_have_changed_since = Param.may_have_changed_since

      let get_obj url =
	let text = Param.text_of_source url in
	let lexbuf = Lexing.from_channel (open_in text) in
	while Lexer.token lexbuf <> Parser.token_start do () done;
	let ref, (lf, pre, args) = Parser.start Lexer.token lexbuf in
	Param.wrapper url (lf, pre, args)
    end)	  
