(** Well: union of 2 wells. *)

module Make
 (W1 : Well.T)
 (W2 : Well.T)
 : Well.T =
struct
  type t = W1 of W1.t | W2 of W2.t 
      
  let kinds cs =
    W1.kinds cs @ W2.kinds cs

  let action_args w =
    match w with
    | W1 w -> W1.action_args w
    | W2 w -> W2.action_args w

  let init s  cs =
    match s with
    | W1 s -> W1.init s  cs
    | W2 s -> W2.init s  cs

  let update s  cs =
    match s with
    | W1 s -> W1.update s  cs
    | W2 s -> W2.update s  cs

  let final s  cs =
    match s with
    | W1 s -> W1.final s  cs
    | W2 s -> W2.final s  cs

  let move s  new_url =
    match s with
    | W1 s -> W1.move s  new_url
    | W2 s -> W2.move s  new_url

  let make url =
    try W1 (W1.make url)
    with _ -> W2 (W2.make url)

  let url s  =
    match s with
    | W1 s -> W1.url s 
    | W2 s -> W2.url s 

end
